"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.MintTokensWidget = void 0;
const react_1 = require("@chakra-ui/react");
const yup_1 = require("@hookform/resolvers/yup");
const yup = __importStar(require("yup"));
const react_hook_form_1 = require("react-hook-form");
const FormControlWithError_1 = require("../form/FormControlWithError");
const react_async_hook_1 = require("react-async-hook");
const wallet_adapter_react_1 = require("@solana/wallet-adapter-react");
const react_2 = require("@strata-foundation/react");
const react_3 = __importDefault(require("react"));
const spl_utils_1 = require("@strata-foundation/spl-utils");
const react_4 = require("react");
const bn_js_1 = __importDefault(require("bn.js"));
const validationSchema = yup.object({
    number: yup.number().typeError("Must be a number").required().positive(),
});
function round(num, decimals) {
    //@ts-ignore
    return +(Math.round(num + "e+" + decimals) + "e-" + decimals);
}
const MintTokensWidget = ({ values, mint, mintKey, }) => {
    const formProps = (0, react_hook_form_1.useForm)({
        resolver: (0, yup_1.yupResolver)(validationSchema),
        defaultValues: values,
    });
    const { register, handleSubmit, watch, formState: { errors, isSubmitting }, } = formProps;
    const { provider, awaitingApproval } = (0, react_2.useProvider)();
    const { publicKey } = (0, wallet_adapter_react_1.useWallet)();
    const { execute, loading, error } = (0, react_async_hook_1.useAsyncCallback)((values) => __awaiter(void 0, void 0, void 0, function* () {
        const factor = (new bn_js_1.default(10).pow(new bn_js_1.default(mint.decimals)));
        yield (0, spl_utils_1.createAtaAndMint)(provider, mintKey, ((new bn_js_1.default(Number(values.number))).mul(factor)).toNumber());
    }));
    const hasAuthority = publicKey && mint && mint.mintAuthority && mint.mintAuthority.equals(publicKey);
    const supply = (0, react_4.useMemo)(() => {
        return mint ? Number(mint.supply) / Math.pow(10, mint.decimals) : 0;
    }, [mint]);
    const onSubmit = (values) => __awaiter(void 0, void 0, void 0, function* () {
        yield execute(values);
    });
    const toMintNum = watch("number", "0");
    return (react_3.default.createElement(react_1.Box, { bgColor: "white", borderRadius: "8px", w: "full" },
        react_3.default.createElement(react_1.Box, { padding: "20px" },
            react_3.default.createElement(react_1.Text, { fontSize: "xl", color: "black.500", fontWeight: "bold", paddingBottom: "10px" }, "Mint Tokens"),
            react_3.default.createElement(react_hook_form_1.FormProvider, Object.assign({}, formProps),
                react_3.default.createElement("form", { onSubmit: handleSubmit(onSubmit), style: { width: "100%" } },
                    react_3.default.createElement(react_1.VStack, { spacing: 8, w: "full" },
                        react_3.default.createElement(FormControlWithError_1.FormControlWithError, { id: "number", help: `Add new tokens into circulation. There are currently ${supply} tokens`, errors: errors },
                            react_3.default.createElement(react_1.Input, Object.assign({}, register("number")))),
                        react_3.default.createElement(react_1.Text, { color: "gray.500", fontSize: "sm", alignSelf: "flex-start", marginTop: "0.5em !important", fontWeight: "bold" },
                            "New Token Amount: ",
                            Number(toMintNum) ? round(supply + Number(toMintNum), mint.decimals) : supply),
                        error && react_3.default.createElement(react_1.Alert, { status: "error" }, error.toString()),
                        react_3.default.createElement(react_1.Button, { isDisabled: !hasAuthority, type: "submit", alignSelf: "flex-end", colorScheme: "primary", isLoading: isSubmitting || loading, loadingText: awaitingApproval ? "Awaiting Approval" : "Loading" }, !hasAuthority ? "Not mint authority" : "Mint")))))));
};
exports.MintTokensWidget = MintTokensWidget;
//# sourceMappingURL=MintTokensWidget.js.map