"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.EditMetadataForm = void 0;
// @ts-nocheck
const react_1 = require("@chakra-ui/react");
const yup_1 = require("@hookform/resolvers/yup");
const wallet_adapter_react_1 = require("@solana/wallet-adapter-react");
const wallet_adapter_react_ui_1 = require("@solana/wallet-adapter-react-ui");
const web3_js_1 = require("@solana/web3.js");
const react_2 = require("@strata-foundation/react");
const router_1 = require("next/router");
const react_3 = __importStar(require("react"));
const react_async_hook_1 = require("react-async-hook");
const react_hook_form_1 = require("react-hook-form");
const yup = __importStar(require("yup"));
const FormControlWithError_1 = require("./FormControlWithError");
const MintSelect_1 = require("./MintSelect");
const TokenMetadataInputs_1 = require("./TokenMetadataInputs");
const validationSchema = yup.object({
    mint: yup.string().required(),
    image: yup.mixed(),
    name: yup.string().required().min(2),
    description: yup.string(),
    symbol: yup.string().min(2).max(10),
    externalUrl: yup.string(),
});
function editMetadata(tokenMetadataSdk, values, wallet) {
    return __awaiter(this, void 0, void 0, function* () {
        const mintKey = new web3_js_1.PublicKey(values.mint);
        const uri = yield tokenMetadataSdk.uploadMetadata({
            name: values.name,
            symbol: values.symbol,
            description: values.description,
            image: values.image,
            mint: mintKey,
            externalUrl: values.externalUrl,
        }, wallet);
        function getMetadata(mint) {
            return __awaiter(this, void 0, void 0, function* () {
                const a = yield web3_js_1.PublicKey.findProgramAddress([
                    Buffer.from("metadata"),
                    new web3_js_1.PublicKey("metaqbxxUerdq28cj1RbAWkYQm3ybzjb6a8bt518x1s").toBuffer(),
                    mint.toBuffer(),
                ], new web3_js_1.PublicKey("metaqbxxUerdq28cj1RbAWkYQm3ybzjb6a8bt518x1s"));
                return [a[0].toBase58(), a[1]];
            });
        }
        const instructions = [];
        const signers = [];
        const metadata = yield mpl_token_metadata_1.Metadata.getPDA(mintKey);
        const data = ({
            // Max name len 32
            name: values.name.substring(0, 32),
            symbol: values.symbol.substring(0, 10),
            uri,
            sellerFeeBasisPoints: 0,
            // @ts-ignore
            creators: null,
            // @ts-ignore
            collection: null,
            // @ts-ignore
            uses: null,
        });
        if (yield tokenMetadataSdk.provider.connection.getAccountInfo(metadata)) {
            const { instructions: metaInstrs, signers: metaSigners } = yield tokenMetadataSdk.updateMetadataInstructions({
                metadata,
                data,
            });
            instructions.push(...metaInstrs);
            signers.push(...metaSigners);
        }
        else {
            const { instructions: metaInstrs, signers: metaSigners } = yield tokenMetadataSdk.createMetadataInstructions({
                mint: mintKey,
                data,
            });
            instructions.push(...metaInstrs);
            signers.push(...metaSigners);
        }
        yield tokenMetadataSdk.sendInstructions(instructions, signers);
        return mintKey;
    });
}
const EditMetadataForm = ({ values, onComplete, }) => {
    const formProps = (0, react_hook_form_1.useForm)({
        resolver: (0, yup_1.yupResolver)(validationSchema),
        defaultValues: values,
    });
    const { register, handleSubmit, watch, setValue, formState: { errors, isSubmitting }, } = formProps;
    const wallet = (0, wallet_adapter_react_1.useWallet)();
    const { awaitingApproval } = (0, react_2.useProvider)();
    const { execute, loading, error } = (0, react_async_hook_1.useAsyncCallback)(editMetadata);
    const { tokenMetadataSdk } = (0, react_2.useStrataSdks)();
    const { visible, setVisible } = (0, wallet_adapter_react_ui_1.useWalletModal)();
    const { connected, publicKey } = (0, wallet_adapter_react_1.useWallet)();
    const router = (0, router_1.useRouter)();
    const { mintKey: mintKeyRaw } = router.query;
    const { mint, name, symbol } = watch();
    const mintKey = (0, react_2.usePublicKey)(mint);
    const mintAcc = (0, react_2.useMint)(mintKey);
    const { metadata, data, image } = (0, react_2.useTokenMetadata)(mintKey);
    const insufficientAuthority = (metadata === null || metadata === void 0 ? void 0 : metadata.updateAuthority)
        ? metadata.updateAuthority != (publicKey === null || publicKey === void 0 ? void 0 : publicKey.toBase58())
        : false;
    (0, react_3.useEffect)(() => {
        if (mint) {
            setValue("mint", mint);
        }
    }, [mint, router, setValue]);
    (0, react_3.useEffect)(() => {
        if (mintKeyRaw) {
            const mintKey = new web3_js_1.PublicKey(mintKeyRaw);
            setValue("mint", mintKey.toString());
        }
    }, [setValue, mintKeyRaw]);
    (0, react_3.useEffect)(() => {
        if (metadata && data && image && !(isSubmitting || loading)) {
            setValue("name", metadata.name);
            setValue("symbol", metadata.symbol);
            setValue("description", (data === null || data === void 0 ? void 0 : data.description) || "");
            setValue("externalUrl", (data === null || data === void 0 ? void 0 : data.external_url) || "");
            (() => __awaiter(void 0, void 0, void 0, function* () {
                const imageFile = yield getFileFromUrl(image, "image");
                if (imageFile)
                    setValue("image", imageFile);
            }))();
        }
    }, [setValue, metadata, data, image]);
    const onSubmit = (values) => __awaiter(void 0, void 0, void 0, function* () {
        yield execute(tokenMetadataSdk, values, wallet);
        onComplete && onComplete(new web3_js_1.PublicKey(values.mint));
    });
    return (react_3.default.createElement(react_1.Flex, { position: "relative", w: "100%" },
        !connected && (react_3.default.createElement(react_1.Flex, { position: "absolute", w: "full", h: "full", zIndex: "1", flexDirection: "column" },
            react_3.default.createElement(react_1.Flex, { justifyContent: "center" },
                react_3.default.createElement(react_1.Button, { colorScheme: "orange", variant: "outline", onClick: () => setVisible(!visible) }, "Connect Wallet")),
            react_3.default.createElement(react_1.Flex, { w: "full", h: "full", bg: "white", opacity: "0.6" }))),
        react_3.default.createElement(react_hook_form_1.FormProvider, Object.assign({}, formProps),
            react_3.default.createElement("form", { onSubmit: handleSubmit(onSubmit), style: { width: "100%" } },
                react_3.default.createElement(react_1.VStack, { spacing: 8, w: "full" },
                    react_3.default.createElement(FormControlWithError_1.FormControlWithError, { id: "mint", help: "The mint/token id of the token whose metadata you wish to update.", label: "Mint", errors: errors },
                        name && (react_3.default.createElement(react_1.Text, { color: "gray.400", size: "sm" },
                            name,
                            " (",
                            symbol,
                            ")")),
                        react_3.default.createElement(MintSelect_1.MintSelect, { value: watch("mint") || "", onChange: (s) => setValue("mint", s) })),
                    react_3.default.createElement(TokenMetadataInputs_1.TokenMetadataInputs, { entityName: "Token" }),
                    react_3.default.createElement(FormControlWithError_1.FormControlWithError, { id: "symbol", help: "A less than 10 character symbol for this token.", label: "Short Name", errors: errors },
                        react_3.default.createElement(react_1.Input, Object.assign({}, register("symbol")))),
                    mintAcc && mintAcc.decimals == 0 && (react_3.default.createElement(FormControlWithError_1.FormControlWithError, { id: "externalUrl", help: "An optional URL that will be rendered in some wallets and explorers", label: "External URL", errors: errors },
                        react_3.default.createElement(react_1.Input, Object.assign({}, register("externalUrl"))))),
                    error && react_3.default.createElement(react_1.Alert, { status: "error" }, error.toString()),
                    react_3.default.createElement(react_1.Button, { isDisabled: insufficientAuthority, type: "submit", alignSelf: "flex-end", colorScheme: "primary", isLoading: isSubmitting || loading, loadingText: awaitingApproval ? "Awaiting Approval" : "Loading" }, insufficientAuthority ? "You do not hold the update authority" : metadata ? "Update Metadata" : "Create Metadata"))))));
};
exports.EditMetadataForm = EditMetadataForm;
const getFileFromUrl = (url, name, defaultType = "image/jpeg") => __awaiter(void 0, void 0, void 0, function* () {
    if (!url) {
        return undefined;
    }
    const data = yield fetch(url, { cache: "no-cache" });
    const blob = yield data.blob();
    if (!blob.type.includes("image"))
        return undefined;
    const fileName = `${name}${blob.type === defaultType ? ".jpeg" : ".png"}`;
    const file = new File([blob], fileName, { type: blob.type || defaultType });
    return file;
});
//# sourceMappingURL=EditMetadataForm.js.map