"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.FullyManagedForm = void 0;
const react_1 = require("@chakra-ui/react");
const yup_1 = require("@hookform/resolvers/yup");
const wallet_adapter_react_1 = require("@solana/wallet-adapter-react");
const web3_js_1 = require("@solana/web3.js");
const react_2 = require("@strata-foundation/react");
const spl_token_bonding_1 = require("@strata-foundation/spl-token-bonding");
const spl_token_collective_1 = require("@strata-foundation/spl-token-collective");
const router_1 = require("next/router");
const react_3 = __importDefault(require("react"));
const react_async_hook_1 = require("react-async-hook");
const react_hook_form_1 = require("react-hook-form");
const yup = __importStar(require("yup"));
const marketplaceSdkContext_1 = require("../..//contexts/marketplaceSdkContext");
const routes_1 = require("../../utils/routes");
const FormControlWithError_1 = require("./FormControlWithError");
const MintSelect_1 = require("./MintSelect");
const TokenMetadataInputs_1 = require("./TokenMetadataInputs");
const Disclosures_1 = require("./Disclosures");
const RadioCard_1 = require("./RadioCard");
const RoyaltiesInputs_1 = require("./RoyaltiesInputs");
const wallet_adapter_react_ui_1 = require("@solana/wallet-adapter-react-ui");
const validationSchema = yup.object({
    mint: yup.string().required(),
    image: yup.mixed().required(),
    name: yup.string().required().min(2),
    description: yup.string().required().min(2),
    symbol: yup.string().required().min(2),
    startingPrice: yup.number().required().min(0),
    isAntiBot: yup.boolean(),
    isSocial: yup.boolean(),
    sellBaseRoyaltyPercentage: yup.number().required(),
    buyBaseRoyaltyPercentage: yup.number().required(),
    sellTargetRoyaltyPercentage: yup.number().required(),
    buyTargetRoyaltyPercentage: yup.number().required(),
    disclosures: Disclosures_1.disclosuresSchema,
});
function createFullyManaged(marketplaceSdk, values) {
    return __awaiter(this, void 0, void 0, function* () {
        const mint = new web3_js_1.PublicKey(values.mint);
        const tokenCollectiveSdk = marketplaceSdk.tokenCollectiveSdk;
        const tokenBondingSdk = tokenCollectiveSdk.splTokenBondingProgram;
        const targetMintKeypair = web3_js_1.Keypair.generate();
        let k = 0;
        switch (values.curveType) {
            case "utility":
                k = 0.5;
                break;
            case "stable":
                k = 1;
                break;
            case "aggressive":
                k = 2;
                break;
        }
        const c = values.startingPrice * (k + 1);
        let config = new spl_token_bonding_1.TimeDecayExponentialCurveConfig({
            c,
            k0: k,
            k1: k,
            d: 1,
            interval: 2 * 60 * 60, // 2 hours
        });
        if (values.isAntiBot) {
            config = new spl_token_bonding_1.TimeCurveConfig()
                .addCurve(0, new spl_token_bonding_1.TimeDecayExponentialCurveConfig({
                c,
                k0: 0,
                k1: 0,
                d: 1,
                interval: 0,
            }))
                .addCurve(30 * 60, // 30 minutes
            new spl_token_bonding_1.TimeDecayExponentialCurveConfig({
                c,
                k0: 0,
                k1: k,
                d: 0.5,
                interval: 1.5 * 60 * 60, // 1.5 hours
            }));
        }
        const curveOut = yield tokenBondingSdk.initializeCurveInstructions({
            config,
        });
        const bondingOpts = {
            baseMint: mint,
            buyBaseRoyaltyPercentage: values.buyBaseRoyaltyPercentage,
            buyTargetRoyaltyPercentage: values.buyTargetRoyaltyPercentage,
            sellBaseRoyaltyPercentage: values.sellBaseRoyaltyPercentage,
            sellTargetRoyaltyPercentage: values.sellTargetRoyaltyPercentage,
            curve: curveOut.output.curve,
            targetMint: targetMintKeypair.publicKey,
            targetMintDecimals: 9,
        };
        const uri = yield tokenCollectiveSdk.splTokenMetadata.uploadMetadata({
            name: values.name,
            symbol: values.symbol,
            description: values.description,
            image: values.image,
            mint: targetMintKeypair.publicKey,
        });
        const metadata = ({
            // Max name len 32
            name: values.name.substring(0, 32),
            symbol: values.symbol.substring(0, 10),
            uri,
            sellerFeeBasisPoints: 0,
            creators: null,
            collection: null,
            uses: null,
        });
        if (values.isSocial) {
            const bondingOut = yield tokenCollectiveSdk.createSocialTokenInstructions({
                mint,
                tokenBondingParams: bondingOpts,
                owner: tokenCollectiveSdk.wallet.publicKey,
                targetMintKeypair,
                metadata,
            });
            yield tokenCollectiveSdk.executeBig(Promise.resolve({
                output: null,
                instructions: [curveOut.instructions, ...bondingOut.instructions],
                signers: [curveOut.signers, ...bondingOut.signers],
            }));
        }
        else {
            const metaOut = yield marketplaceSdk.createMetadataForBondingInstructions({
                targetMintKeypair,
                metadataUpdateAuthority: tokenCollectiveSdk.wallet.publicKey,
                metadata,
                decimals: bondingOpts.targetMintDecimals,
            });
            const bondingOut = yield tokenBondingSdk.createTokenBondingInstructions(bondingOpts);
            yield tokenBondingSdk.executeBig(Promise.resolve({
                output: null,
                instructions: [
                    [...curveOut.instructions, ...metaOut.instructions],
                    bondingOut.instructions,
                ],
                signers: [
                    [...curveOut.signers, ...metaOut.signers],
                    bondingOut.signers,
                ],
            }));
        }
        return targetMintKeypair.publicKey;
    });
}
const FullyManagedForm = () => {
    const formProps = (0, react_hook_form_1.useForm)({
        // @ts-ignore
        resolver: (0, yup_1.yupResolver)(validationSchema),
    });
    const { register, handleSubmit, setValue, formState: { errors, isSubmitting }, watch, } = formProps;
    const { connected, publicKey } = (0, wallet_adapter_react_1.useWallet)();
    const { visible, setVisible } = (0, wallet_adapter_react_ui_1.useWalletModal)();
    const { awaitingApproval } = (0, react_2.useProvider)();
    const { execute, loading, error } = (0, react_async_hook_1.useAsyncCallback)(createFullyManaged);
    const { marketplaceSdk } = (0, marketplaceSdkContext_1.useMarketplaceSdk)();
    const router = (0, router_1.useRouter)();
    const onSubmit = (values) => __awaiter(void 0, void 0, void 0, function* () {
        const mintKey = yield execute(marketplaceSdk, values);
        router.push((0, routes_1.route)(routes_1.routes.tokenAdmin, {
            mintKey: mintKey.toBase58(),
        }), undefined, { shallow: true });
    });
    const { name = "", symbol = "", isSocial, mint, curveType } = watch();
    const mintKey = (0, react_2.usePublicKey)(mint);
    const { result: collectiveKey } = (0, react_async_hook_1.useAsync)((mint) => __awaiter(void 0, void 0, void 0, function* () { return mint ? spl_token_collective_1.SplTokenCollective.collectiveKey(new web3_js_1.PublicKey(mint)) : undefined; }), [mint]);
    const { info: collective } = (0, react_2.useCollective)(collectiveKey && collectiveKey[0]);
    const tokenBondingSettings = collective === null || collective === void 0 ? void 0 : collective.config.claimedTokenBondingSettings;
    const { metadata: baseMetadata, error: baseMetadataError, loading: baseMetadataLoading, } = (0, react_2.useTokenMetadata)(mintKey);
    const { getRootProps, getRadioProps } = (0, react_1.useRadioGroup)({
        name: "curveType",
        onChange: (option) => setValue("curveType", option),
    });
    const group = getRootProps();
    const curveOptions = [
        {
            value: "aggressive",
            heading: "Aggressive",
            illustration: "/aggressive.svg",
            helpText: "A curve with high price sensitivity. The price raises quickly when people buy, and lowers quickly when they sell. This is best suited for speculative use cases.",
        },
        {
            value: "stable",
            heading: "Stable",
            illustration: "/stable.svg",
            helpText: "A curve with medium price sensitivity. This curve changes price at a constant rate, achieving a balance between aggressive and utility curves.",
        },
        {
            value: "utility",
            heading: "Utility",
            illustration: "/utility.svg",
            helpText: "A curve with a price sensitivity that starts high and lowers with purchases. This curve is best suited for utility use cases, as it rewards early adopters and scales the supply so that the token can be exchanged for goods/services.",
        },
    ];
    return (react_3.default.createElement(react_1.Flex, { position: "relative" },
        !connected && (react_3.default.createElement(react_1.Flex, { position: "absolute", w: "full", h: "full", zIndex: "1", flexDirection: "column" },
            react_3.default.createElement(react_1.Flex, { justifyContent: "center" },
                react_3.default.createElement(react_1.Button, { colorScheme: "orange", variant: "outline", onClick: () => setVisible(!visible) }, "Connect Wallet")),
            react_3.default.createElement(react_1.Flex, { w: "full", h: "full", bg: "white", opacity: "0.6" }))),
        react_3.default.createElement(react_hook_form_1.FormProvider, Object.assign({}, formProps),
            react_3.default.createElement("form", { onSubmit: handleSubmit(onSubmit) },
                react_3.default.createElement(react_1.VStack, { spacing: 8, mt: !connected ? 12 : 0 },
                    react_3.default.createElement(TokenMetadataInputs_1.TokenMetadataInputs, { entityName: "token" }),
                    react_3.default.createElement(FormControlWithError_1.FormControlWithError, { id: "symbol", help: "The symbol for this token, ex: SOL", label: "Symbol", errors: errors },
                        react_3.default.createElement(react_1.Input, Object.assign({}, register("symbol")))),
                    react_3.default.createElement(FormControlWithError_1.FormControlWithError, { id: "curveType", label: "Price Sensitivity", errors: errors },
                        react_3.default.createElement(react_1.Stack, Object.assign({}, group, { direction: { base: "column", md: "row" }, justifyContent: "center", alignItems: { base: "center", md: "normal" } }), curveOptions.map(({ value, heading, illustration, helpText }) => {
                            const radio = getRadioProps({ value });
                            return (react_3.default.createElement(RadioCard_1.RadioCardWithAffordance, Object.assign({ key: value }, radio),
                                react_3.default.createElement(react_1.Flex, { h: "full", direction: { base: "row", md: "column" }, textAlign: { base: "left", md: "center" } },
                                    react_3.default.createElement(react_1.Flex, { justifyContent: "center", alignItems: "center", flexShrink: 0 },
                                        react_3.default.createElement(react_1.Image, { src: illustration, alt: `${value}-illustration`, height: "70px", width: "100%" })),
                                    react_3.default.createElement(react_1.Flex, { flexGrow: 1, h: "full", direction: "column", alignItems: { base: "start", md: "center" }, justifyContent: { base: "center", md: "initial" } },
                                        react_3.default.createElement(react_1.Text, { fontWeight: "bold", fontSize: "md", pt: { base: 0, md: 4 } }, heading),
                                        react_3.default.createElement(react_1.Flex, { w: "full", flexGrow: { base: 0, md: 1 }, alignItems: { base: "start", md: "center" } },
                                            react_3.default.createElement(react_1.Text, { fontSize: "xs", color: "gray.500" }, helpText))))));
                        }))),
                    react_3.default.createElement(FormControlWithError_1.FormControlWithError, { id: "isSocial", help: `If this is a social token, it will be associated with your wallet. This means applications like Wum.bo will be able to discover this token by looking up your wallet, which may be associated with your twitter handle, .sol domain, or any other web3 applications. A social token can be part of a network of other social tokens: a collective.`, label: "Social Token?", errors: errors },
                        react_3.default.createElement(react_1.Switch, Object.assign({}, register("isSocial")))),
                    react_3.default.createElement(FormControlWithError_1.FormControlWithError, { id: "mint", help: `The mint that should be used to purchase this token, example BQpGv6LVWG1JRm1NdjerNSFdChMdAULJr3x9t2Swpump for Fomo3d.fun`, label: "Mint", errors: errors },
                        react_3.default.createElement(MintSelect_1.MintSelect, { value: watch("mint"), onChange: (s) => setValue("mint", s) }),
                        " "),
                    react_3.default.createElement(FormControlWithError_1.FormControlWithError, { id: "startingPrice", help: "The starting price of the token. The price will increase as more tokens are purchased", label: "Starting Price", errors: errors },
                        react_3.default.createElement(react_1.Input, Object.assign({ type: "number", min: 0, step: 0.0000000001 }, register("startingPrice")))),
                    react_3.default.createElement(FormControlWithError_1.FormControlWithError, { id: "isAntiBot", help: `Enable anti botting measures. This will keep bots from profiting by frontrunning your token while the price is low. Your tokens true pricing will take 2 hours to come into effect`, label: "Enable Anti Bot Measures?", errors: errors },
                        react_3.default.createElement(react_1.Switch, Object.assign({}, register("isAntiBot")))),
                    react_3.default.createElement(RoyaltiesInputs_1.RoyaltiesInputs, { symbol: symbol, baseSymbol: baseMetadata === null || baseMetadata === void 0 ? void 0 : baseMetadata.data.symbol, register: register, minBuyTargetRoyaltyPercentage: tokenBondingSettings === null || tokenBondingSettings === void 0 ? void 0 : tokenBondingSettings.minBuyTargetRoyaltyPercentage, maxBuyTargetRoyaltyPercentage: tokenBondingSettings === null || tokenBondingSettings === void 0 ? void 0 : tokenBondingSettings.maxBuyTargetRoyaltyPercentage, minSellTargetRoyaltyPercentage: tokenBondingSettings === null || tokenBondingSettings === void 0 ? void 0 : tokenBondingSettings.minSellTargetRoyaltyPercentage, maxSellTargetRoyaltyPercentage: tokenBondingSettings === null || tokenBondingSettings === void 0 ? void 0 : tokenBondingSettings.maxSellTargetRoyaltyPercentage, minBuyBaseRoyaltyPercentage: tokenBondingSettings === null || tokenBondingSettings === void 0 ? void 0 : tokenBondingSettings.minBuyBaseRoyaltyPercentage, maxBuyBaseRoyaltyPercentage: tokenBondingSettings === null || tokenBondingSettings === void 0 ? void 0 : tokenBondingSettings.maxBuyBaseRoyaltyPercentage, minSellBaseRoyaltyPercentage: tokenBondingSettings === null || tokenBondingSettings === void 0 ? void 0 : tokenBondingSettings.minSellBaseRoyaltyPercentage, maxSellBaseRoyaltyPercentage: tokenBondingSettings === null || tokenBondingSettings === void 0 ? void 0 : tokenBondingSettings.maxSellBaseRoyaltyPercentage }),
                    react_3.default.createElement(Disclosures_1.Disclosures, { fees: 0 }),
                    error && (react_3.default.createElement(react_1.Alert, { status: "error" },
                        react_3.default.createElement(react_1.Alert, { status: "error" }, error.toString()))),
                    react_3.default.createElement(react_1.Button, { type: "submit", alignSelf: "flex-end", colorScheme: "primary", isLoading: isSubmitting || loading, loadingText: awaitingApproval ? "Awaiting Approval" : "Loading" }, "Create Token"))))));
};
exports.FullyManagedForm = FullyManagedForm;
//# sourceMappingURL=FullyManagedForm.js.map