"use strict";
var __createBinding = (this && this.__createBinding) || (Object.create ? (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    var desc = Object.getOwnPropertyDescriptor(m, k);
    if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
      desc = { enumerable: true, get: function() { return m[k]; } };
    }
    Object.defineProperty(o, k2, desc);
}) : (function(o, m, k, k2) {
    if (k2 === undefined) k2 = k;
    o[k2] = m[k];
}));
var __setModuleDefault = (this && this.__setModuleDefault) || (Object.create ? (function(o, v) {
    Object.defineProperty(o, "default", { enumerable: true, value: v });
}) : function(o, v) {
    o["default"] = v;
});
var __importStar = (this && this.__importStar) || function (mod) {
    if (mod && mod.__esModule) return mod;
    var result = {};
    if (mod != null) for (var k in mod) if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);
    __setModuleDefault(result, mod);
    return result;
};
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.TokenAuthorityForm = void 0;
const react_1 = require("@chakra-ui/react");
const yup_1 = require("@hookform/resolvers/yup");
const yup = __importStar(require("yup"));
const react_hook_form_1 = require("react-hook-form");
const web3_js_1 = require("@solana/web3.js");
const FormControlWithError_1 = require("./FormControlWithError");
const react_async_hook_1 = require("react-async-hook");
const wallet_adapter_react_1 = require("@solana/wallet-adapter-react");
const react_2 = require("@strata-foundation/react");
const react_3 = __importStar(require("react"));
const spl_token_1 = require("@solana/spl-token");
const validationSchema = yup.object({
    mintAuthority: yup.string(),
    freezeAuthority: yup.string(),
});
function mintAuthChanged(mint, values) {
    return !!((mint && mint.mintAuthority && mint.mintAuthority.toString() != values.mintAuthority) ||
        (mint && !mint.mintAuthority && values.mintAuthority != ""));
}
function freezeAuthChanged(mint, values) {
    return !!((mint && mint.freezeAuthority && mint.freezeAuthority.toString() != values.freezeAuthority) ||
        (mint && !mint.freezeAuthority && values.freezeAuthority != ""));
}
function validateAuthorities(mint, values, hasMintAuth, hasFreezeAuth) {
    if (mintAuthChanged(mint, values)) {
        if (!hasMintAuth) {
            return "You need the mint authority to change it";
        }
    }
    if (freezeAuthChanged(mint, values)) {
        if (!hasFreezeAuth) {
            return "You need the freeze authority to change it";
        }
    }
    if (mintAuthChanged(mint, values) || freezeAuthChanged(mint, values))
        return true;
    return "No changes made";
}
const TokenAuthorityForm = ({ values, metadata, mint, mintKey, }) => {
    const { tokenBondingSdk, fungibleEntanglerSdk } = (0, react_2.useStrataSdks)();
    const formProps = (0, react_hook_form_1.useForm)({
        resolver: (0, yup_1.yupResolver)(validationSchema),
        defaultValues: values,
    });
    const { register, handleSubmit, watch, setValue, formState: { errors, isSubmitting }, } = formProps;
    const { awaitingApproval } = (0, react_2.useProvider)();
    (0, react_3.useEffect)(() => {
        var _a, _b;
        setValue("mintAuthority", ((_a = mint === null || mint === void 0 ? void 0 : mint.mintAuthority) === null || _a === void 0 ? void 0 : _a.toString()) || "");
        setValue("freezeAuthority", ((_b = mint === null || mint === void 0 ? void 0 : mint.freezeAuthority) === null || _b === void 0 ? void 0 : _b.toString()) || "");
    }, [setValue, mint]);
    const { connected, publicKey } = (0, wallet_adapter_react_1.useWallet)();
    const hasMintAuthority = !!(mint && publicKey && mint.mintAuthority && mint.mintAuthority.equals(publicKey));
    const hasFreezeAuthority = !!(mint && publicKey && mint.freezeAuthority && mint.freezeAuthority.equals(publicKey));
    const hasAnyAuth = hasMintAuthority || hasFreezeAuthority;
    const { execute, loading, error } = (0, react_async_hook_1.useAsyncCallback)((mint, values) => __awaiter(void 0, void 0, void 0, function* () {
        const valid = validateAuthorities(mint, values, hasMintAuthority, hasFreezeAuthority);
        if (typeof (valid) == "string")
            throw valid;
        const instructions = [];
        if (mintAuthChanged(mint, values)) {
            instructions.push(spl_token_1.Token.createSetAuthorityInstruction(spl_token_1.TOKEN_PROGRAM_ID, mintKey, values.mintAuthority == "" ? null : new web3_js_1.PublicKey(values.mintAuthority), "MintTokens", publicKey, []));
        }
        if (freezeAuthChanged(mint, values)) {
            instructions.push(spl_token_1.Token.createSetAuthorityInstruction(spl_token_1.TOKEN_PROGRAM_ID, mintKey, values.freezeAuthority == "" ? null : new web3_js_1.PublicKey(values.freezeAuthority), "FreezeAccount", publicKey, []));
        }
        tokenBondingSdk.sendInstructions(instructions, []);
    }));
    const onSubmit = (values) => __awaiter(void 0, void 0, void 0, function* () {
        yield execute(mint, values);
    });
    return (react_3.default.createElement(react_1.Box, { bgColor: "white", borderRadius: "8px", w: "full" },
        react_3.default.createElement(react_1.Box, { padding: "20px" },
            react_3.default.createElement(react_hook_form_1.FormProvider, Object.assign({}, formProps),
                react_3.default.createElement("form", { onSubmit: handleSubmit(onSubmit), style: { width: "100%" } },
                    react_3.default.createElement(react_1.VStack, { spacing: 8, w: "full" },
                        react_3.default.createElement(FormControlWithError_1.FormControlWithError, { id: "mintAuthority", label: "Mint Authority", errors: errors },
                            react_3.default.createElement(react_1.Input, Object.assign({}, register("mintAuthority")))),
                        react_3.default.createElement(FormControlWithError_1.FormControlWithError, { id: "freezeAuthority", label: "Freeze Authority", errors: errors },
                            react_3.default.createElement(react_1.Input, Object.assign({}, register("freezeAuthority")))),
                        error && react_3.default.createElement(react_1.Alert, { status: "error" }, error.toString()),
                        react_3.default.createElement(react_1.Button, { isDisabled: !hasAnyAuth, type: "submit", alignSelf: "flex-end", colorScheme: "primary", isLoading: isSubmitting || loading, loadingText: awaitingApproval ? "Awaiting Approval" : "Loading" }, !hasAnyAuth ? "You do not hold mint or freeze authority" : "Update Authorities")))))));
};
exports.TokenAuthorityForm = TokenAuthorityForm;
//# sourceMappingURL=TokenAuthorityForm.js.map