"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.DynamicPricingCandyMachine = void 0;
const react_1 = require("@chakra-ui/react");
const solana_gateway_react_1 = require("@civic/solana-gateway-react");
const wallet_adapter_react_1 = require("@solana/wallet-adapter-react");
const web3_js_1 = require("@solana/web3.js");
const react_2 = require("@strata-foundation/react");
const bn_js_1 = __importDefault(require("bn.js"));
const react_3 = __importDefault(require("react"));
const react_hot_toast_1 = __importDefault(require("react-hot-toast"));
const hooks_1 = require("../../../hooks");
const BondingPlot_1 = require("../BondingPlot");
const Branding_1 = require("../Branding");
const LbcInfo_1 = require("../LbcInfo");
const LbcStatus_1 = require("../LbcStatus");
const MintButton_1 = require("../MintButton");
const TransactionHistory_1 = require("../TransactionHistory");
const candy_machine_1 = require("./candy-machine");
const CandyMachineInfo_1 = require("./CandyMachineInfo");
const MintedNftNotification_1 = require("./MintedNftNotification");
const utils_1 = require("./utils");
const DynamicPricingCandyMachine = (props) => {
    var _a, _b, _c;
    const { connection } = (0, wallet_adapter_react_1.useConnection)();
    const { publicKey, connected, signTransaction, wallet } = (0, wallet_adapter_react_1.useWallet)();
    const cmState = (0, hooks_1.useCandyMachineInfo)(props.candyMachineId);
    const { candyMachine, isWhitelistUser, isActive, isPresale } = cmState;
    const mintKey = candyMachine === null || candyMachine === void 0 ? void 0 : candyMachine.tokenMint;
    const { info: tokenBonding, loading, error, } = (0, react_2.useTokenBondingFromMint)(mintKey);
    const { price } = (0, hooks_1.useLivePrice)(tokenBonding === null || tokenBonding === void 0 ? void 0 : tokenBonding.publicKey);
    const background = (0, react_1.useColorModeValue)("white", "black.300");
    const onMint = (args) => __awaiter(void 0, void 0, void 0, function* () {
        var _d;
        try {
            (_d = document.getElementById("#identity")) === null || _d === void 0 ? void 0 : _d.click();
            if (connected &&
                (candyMachine === null || candyMachine === void 0 ? void 0 : candyMachine.program) &&
                publicKey &&
                props.candyMachineId) {
                const mint = yield (0, candy_machine_1.mintOneToken)(candyMachine, publicKey, args);
                if (props.onSuccess) {
                    props.onSuccess(mint);
                }
                else {
                    react_hot_toast_1.default.custom((t) => (react_3.default.createElement(MintedNftNotification_1.MintedNftNotification, { mint: mint, onDismiss: () => react_hot_toast_1.default.dismiss(t.id) })), {
                        duration: Infinity,
                    });
                }
            }
        }
        catch (error) {
            let message = error.msg || error.toString() || "Minting failed! Please try again!";
            let heading = "Transaction Failed";
            if (!error.msg) {
                if (!error.message) {
                    message = "Transaction Timeout! Please try again.";
                }
                else if (error.message.indexOf("0x137") != -1) {
                    message = `SOLD OUT!`;
                }
                else if (error.message.indexOf("0x135") != -1) {
                    message = `Insufficient funds to mint. Please fund your wallet.`;
                }
            }
            else {
                if (error.code === 311) {
                    message = `SOLD OUT!`;
                    window.location.reload();
                }
                else if (error.code === 312) {
                    message = `Minting period hasn't started yet.`;
                }
                else if (error.code === 6005 || error.code === 6006) {
                    heading = "Transaction Cancelled";
                    message =
                        "The price moved unfavorably by more than the configured slippage. Change slippage by clicking Advanced Settings";
                }
                else {
                    message = error.toString();
                }
            }
            console.error(error);
            react_hot_toast_1.default.custom((t) => (react_3.default.createElement(react_2.Notification, { show: t.visible, type: "error", heading: heading, message: message, onDismiss: () => react_hot_toast_1.default.dismiss(t.id) })), {
                duration: 120 * 1000,
            });
        }
    });
    const selectedProps = {
        borderBottom: "3px solid #F07733",
    };
    return (react_3.default.createElement(react_1.Tabs, { variant: "unstyled", isLazy: true },
        react_3.default.createElement(react_1.TabList, { borderBottom: "none" },
            react_3.default.createElement(react_1.Tab, { _selected: selectedProps, fontWeight: 600 }, "Mint"),
            tokenBonding && (react_3.default.createElement(react_1.Tab, { _selected: selectedProps, fontWeight: 600 }, "Transactions"))),
        react_3.default.createElement(react_1.TabPanels, null,
            react_3.default.createElement(react_1.TabPanel, { p: 0, pt: 4 },
                typeof window !== "undefined" && react_3.default.createElement(LbcStatus_1.LbcStatus, { tokenBondingKey: tokenBonding === null || tokenBonding === void 0 ? void 0 : tokenBonding.publicKey }),
                react_3.default.createElement(react_1.Box, { zIndex: 1, shadow: "xl", rounded: "lg", p: "16px", pb: "29px", minH: "300px", bg: background },
                    connected && (react_3.default.createElement(react_3.default.Fragment, null,
                        !candyMachine && (react_3.default.createElement(react_1.Center, null,
                            react_3.default.createElement(react_1.Spinner, null))),
                        candyMachine && (react_3.default.createElement(react_1.VStack, { align: "stretch", spacing: 8 },
                            tokenBonding && (react_3.default.createElement(LbcInfo_1.LbcInfo, { price: price, id: tokenBonding.targetMint })),
                            !tokenBonding && react_3.default.createElement(CandyMachineInfo_1.CandyMachineInfo, Object.assign({}, cmState)),
                            (candyMachine === null || candyMachine === void 0 ? void 0 : candyMachine.isActive) &&
                                (candyMachine === null || candyMachine === void 0 ? void 0 : candyMachine.gatekeeper) &&
                                publicKey &&
                                signTransaction ? (
                            // @ts-ignore
                            react_3.default.createElement(solana_gateway_react_1.GatewayProvider, { wallet: {
                                    publicKey: publicKey || new web3_js_1.PublicKey(hooks_1.CANDY_MACHINE_PROGRAM),
                                    //@ts-ignore
                                    signTransaction: wallet.signTransaction,
                                }, gatekeeperNetwork: (_a = candyMachine === null || candyMachine === void 0 ? void 0 : candyMachine.gatekeeper) === null || _a === void 0 ? void 0 : _a.gatekeeperNetwork, 
                                // @ts-ignore
                                clusterUrl: connection._rpcEndpoint, options: { autoShowModal: false } },
                                react_3.default.createElement(MintButton_1.MintButton, { price: price, onMint: onMint, tokenBondingKey: tokenBonding === null || tokenBonding === void 0 ? void 0 : tokenBonding.publicKey, isDisabled: (!isActive && (!isPresale || !isWhitelistUser)) ||
                                        (candyMachine.isWhitelistOnly && !isWhitelistUser), disabledText: candyMachine.isWhitelistOnly && !isWhitelistUser
                                        ? "No Whitelist Token"
                                        : `Mint launches ${(_b = getCountdownDate(candyMachine)) === null || _b === void 0 ? void 0 : _b.toLocaleTimeString()}` }))) : (react_3.default.createElement(MintButton_1.MintButton, { price: price, onMint: onMint, tokenBondingKey: tokenBonding === null || tokenBonding === void 0 ? void 0 : tokenBonding.publicKey, isDisabled: (!isActive && (!isPresale || !isWhitelistUser)) ||
                                    (candyMachine.isWhitelistOnly && !isWhitelistUser), disabledText: candyMachine.isWhitelistOnly && !isWhitelistUser
                                    ? "No Whitelist Token"
                                    : `Mint launches ${(_c = getCountdownDate(candyMachine)) === null || _c === void 0 ? void 0 : _c.toLocaleTimeString()}` })),
                            react_3.default.createElement(Branding_1.Branding, null))))),
                    !connected && (react_3.default.createElement(react_1.Center, null,
                        react_3.default.createElement(react_1.Button, { variant: "outline", colorScheme: "primary", onClick: props.onConnectWallet }, "Connect Wallet"))))),
            react_3.default.createElement(react_1.TabPanel, { p: 0, pt: 4 },
                react_3.default.createElement(react_1.Box, { zIndex: 1, shadow: "xl", rounded: "lg", p: "16px", pb: "29px", minH: "300px", bg: background },
                    react_3.default.createElement(react_1.VStack, { align: "stretch", spacing: 8 },
                        react_3.default.createElement(BondingPlot_1.BondingPlot, { tokenBondingKey: tokenBonding === null || tokenBonding === void 0 ? void 0 : tokenBonding.publicKey }),
                        react_3.default.createElement(TransactionHistory_1.TransactionHistory, { tokenBondingKey: tokenBonding === null || tokenBonding === void 0 ? void 0 : tokenBonding.publicKey })))))));
};
exports.DynamicPricingCandyMachine = DynamicPricingCandyMachine;
const getCountdownDate = (candyMachine) => {
    var _a;
    if (candyMachine.isActive && ((_a = candyMachine.endSettings) === null || _a === void 0 ? void 0 : _a.endSettingType.date)) {
        return (0, utils_1.toDate)(candyMachine.endSettings.number);
    }
    return (0, utils_1.toDate)(candyMachine.goLiveDate
        ? candyMachine.goLiveDate
        : candyMachine.isPresale
            ? new bn_js_1.default(new Date().getTime() / 1000)
            : undefined);
};
//# sourceMappingURL=DynamicPricingCandyMachine.js.map